<?php
/*
	FormMaker & FormPrinter class

	This class helps generate and validate forms as well as make them
	printable in HTML

	Version 1.1.0

	changelog

	1.1.0
		- fix empty label issue

	1.0.9
		- added maxlength (based on crit_array[1]
		- started this log..
*/


class FormMaker {
	public function __construct($form_name, $form_action="") {
		$this->form['form_name'] = $form_name;
		$this->form['form_action'] = $form_action;
	}
	function set_form_name($form_name) {
		$this->form['form_name'] = $form_name;
	}
	function set_form_action($form_action) {
		$this->form['form_action'] = $form_action;
	}
	function set_form_id($id) {
		$this->form['form_id'] = $id;
	}
	function add_element($name, $type, $criteria, $mandatory="") {
		$this->form['fields'][$name]['name'] = $name;
		$this->form['fields'][$name]['type'] = $type;
		$this->form['fields'][$name]['criteria'] = $criteria;
		$this->form['fields'][$name]['mandatory'] = $mandatory;
		if ($criteria == "bool") {
			$this->form['fields'][$name]['value'] = "1";
		} else {
			$this->form['fields'][$name]['value'] = "";
		}
		$this->form['fields'][$name]['css'] = "";
		$this->set_criteria($name, $criteria);
	}
	function del_element($name) {
		unset($this->form['fields'][$name]);
	}
	function import_array($array) {
		$this->form = $array;
	}
    function switch_mandatory($name, $t=false) {
		if (isset($t)) {
			$this->form['fields'][$name]['mandatory'] = true;
		} else {
			$this->form['fields'][$name]['mandatory'] = false;
		}
	}

	function set_display_only($key, $set = TRUE) {
		$this->form['fields'][$key]['display_only'] = $set;
	}

	function set_criteria($key, $criteria) {
		//echo "<pre>$criteria</pre>:";
		$crit_array = explode("|", $criteria, 2);

		// Set maxlength
		if (!empty($crit_array[1]) AND is_valid_id($crit_array[1])) {
			$this->form['fields'][$key]['maxlength'] = $crit_array[1];
		}

		if ($crit_array[0] == "monoselect") {
			$monoselect = explode(":", $crit_array['1']);
			foreach ($monoselect as $k => $v) {
				if (!empty($v)) {
					$monoselect_array = explode("=", $v);
					$new_key = $monoselect_array['0'];
					$test_array[$new_key]['val'] = $monoselect_array[1];
					$this->form['fields'][$key]['val'] = $monoselect_array[1];
				}
			}
			$this->form['fields'][$key]['monoselect'] = $test_array;
			unset($test_array);
		} else if ($crit_array[0] == "multiselect") {
			$crit_array[1] = rtrim($crit_array[1], ":");
			$multiselect = explode(":", $crit_array[1]);
			foreach ($multiselect as $k => $v) {
				$multiselect_array = explode("=", $v);
				$new_key = $multiselect_array['0'];
				$test_array[$new_key]['val'] = $multiselect_array[1];
			}
			$this->form['fields'][$key]['multiselect'] = $test_array;
			unset($test_array);
		}
	}

	function add_desc($key, $desc) {
		$this->form['fields'][$key]['desc'] = $desc;
	}
	function add_input_desc($key, $desc) {
		$this->form['fields'][$key]['input_desc'] = $desc;
	}
	function set_lang($key, $lang) {
		if (!empty($lang)) {
			$this->form['fields'][$key]['lang'] = $lang;
		}
	}
	function set_error($key) {
		if ($this->form['fields'][$key]) {
			$this->form['fields'][$key]['error'] = 1;
			$this->form['fields'][$key]['css'] = "field_error";
			if (empty($this->form['errors'])) {
				$this->form['errors'] = '';
			}
			$this->form['errors']++;
		}
	}
	function set_value($key, $value) {
		if ($this->form['fields'][$key]) {
			$this->form['fields'][$key]['value'] = $value;
			if ($this->form['fields'][$key]['multiselect']) {
				foreach ($this->form['fields'][$key]['multiselect'] as $k => $v) {
					unset($this->form['fields'][$key]['multiselect'][$k]['checked']);
					unset($this->form['fields'][$key]['multiselect'][$k]['selected']);
				}
			} else if ($this->form['fields'][$key]['monoselect']) {
				foreach ($this->form['fields'][$key]['monoselect'] as $k => $v) {
					unset($this->form['fields'][$key]['monoselect'][$k]['checked']);
					unset($this->form['fields'][$key]['monoselect'][$k]['selected']);
				}
			}
			return true;
		} else {
			return false;
		}
	}
	function set_display_name($key, $value) {
		$this->form['fields'][$key]['display_name'] = "$value";
	}
	function set_value_and_validate($key, $value) {
		if ($this->form['fields'][$key]) {
			// TODO should this replace the above ? - NO
			$this->form['fields'][$key]['value'] = $value;

			if ($this->form['fields'][$key]['mandatory'] AND !strlen($value)) {
				$this->form['fields'][$key]['error'] = 1;
				$this->form['fields'][$key]['css'] = "field_empty";
				$this->form['errors']++;
			} else {
				$this->validate($key, $value, $this->form['fields'][$key]['criteria']);
			}
		} else {
			return false;
		}
	}
	function check_post($post_array) {
		foreach ($this->form['fields'] as $k => $v ) {
			if (!empty($v['display_only'])) {
				// Set value, else it gets lost
				$this->form['fields'][$k]['value'] = $post_array[$k];

			} else if (!empty($v['mandatory']) AND !strlen($post_array[$k])) {
				$this->form['fields'][$k]['error'] = 1;
				$this->form['fields'][$k]['css'] = "field_empty";
				$this->form['errors']++;

			} else if (!empty($post_array[$k]) OR is_numeric($post_array[$k])) {
				// if not empty, then validate it
				// copy back the value into to form, for re-display
				$this->form['fields'][$k]['value'] = $post_array[$k];
				// validate the posted value
				$this->validate($k, $post_array[$k], $this->form['fields'][$k]['criteria']);
			// Reset values that don't fit FIXME why ?
			} else {
				// Unset value
				$this->form['fields'][$k]['value'] = "";
			}
		}
	}
	function validate($field, $var, $criteria) {
		/* explode criteria "|"
			['0'] = type
			['1'] = attr
			define types
				- txt_nospace
				- txt_space
				- txt_no_html
				- txt_no_html_no_links
				- numbers_nospace
				- numbers_space
				- monoselect
				- multiselect
				- custom (use after | as regex)
			if (preg_match("#^['0-9']{1,}$#", $number)) { */

		// any letter, number, minium 1 char, no spaces
		$tests['txt_nospace'] = "#^[a-zA-Z0-9]{1,REPLACE}$#";
		$tests['txt_nospace_hyphen'] = "#^[a-zA-Z0-9\-]{1,REPLACE}$#";
		$tests['urls'] = "#^[a-zA-Z0-9\-_]{1,REPLACE}$#";
		$tests['txt_space'] = "#^[a-zA-Z0-9\ ]{1,REPLACE}$#";
		// letters with accents -> \p{L}
		$tests['txt_letters_accents_hyphen'] = "#^[\p{L}\ \-]{1,REPLACE}$#";
		$tests['txt_letters_accents_hyphen_apostrophe'] = "#^[\p{L}\ \-\']{1,REPLACE}$#";
		//$tests['integer'] = "#^[0-9]{1,REPLACE}$#";
		$tests['integer'] = "#^[-]?[0-9]{1,REPLACE}$#";

		$tests['bool'] = "#^[01]?$#i";
		$tests['empty'] = "#^$#";
		// Y/n (case agnostic)
		$tests['yn'] = "#^[YN]{1}$#i";
		# just a 1, true, Y, y
		$tests['checkbox'] = "#^[Y1]{1}|true$#i";
		// Free text, this might be the most common case
		$tests['free'] = "#^(.){1,REPLACE}$#";
		// write your own regex for the occassional special thing
		$tests['custom'] = "REPLACE";

		$crit_array = explode("|", $criteria, 2);

		// certain types can be tested differently
		if ($crit_array[0] == "email") {
			$test_mail = filter_var("$var", FILTER_VALIDATE_EMAIL);
			if (!$test_mail) {
				$this->set_error($field);
			}
		} else if ($crit_array[0] == "one_email_per_line") {
			$test_email_array = explode("\n", $var);
			foreach ($test_email_array as $email) {
				$email = trim($email);
				$email = str_replace(array(" ", ","), "", $email);
				$test_mail = filter_var("$email", FILTER_VALIDATE_EMAIL);
				if (!$test_mail) {
					$this->set_error($field);
				}
			}
		} else if ($crit_array[0] == "full_url") {
			$test_url = filter_var("$var", FILTER_VALIDATE_URL);
			if (!$test_url) {
				$this->set_error($field);
			}

		} else if ($crit_array[0] == "txt_no_html") {
			// Works only for enclosed tags, even non existant tags
			//if (preg_match( "/\/[a-z]*>/i", $var)) {
			// Returns 1 for any tag
			if ($var != strip_tags($var)) {
				$this->set_error($field);
			}

		} else if ($crit_array[0] == "txt_no_html_no_links") {
			// No HTML AND NO LINKS
			if ($var != strip_tags($var) OR preg_match("#https?://#", $var)) {
				$this->set_error($field);
			}

		} else if ($crit_array[0] == "ip") {
			$test_ip = filter_var("$var", FILTER_VALIDATE_IP);
			if (!$test_ip) {
				$this->set_error($field);
			}

		} else if ($crit_array[0] == "ipv6") {
			$test_ip = filter_var("$var", FILTER_VALIDATE_IP, FILTER_FLAG_IPV6);
			if (!$test_ip) {
				$this->set_error($field);
			}

		} else if ($crit_array[0] == "ipv4and6") {
			$test_ip4 = filter_var("$var", FILTER_VALIDATE_IP, FILTER_FLAG_IPV4);
			$test_ip6 = filter_var("$var", FILTER_VALIDATE_IP, FILTER_FLAG_IPV6);
			if (!$test_ip4 AND !$test_ip6) {
				$this->set_error($field);
			}

		} else if ($crit_array[0] == "confirm") {
			$verifyfield = $crit_array[1];
			if (!$this->form['fields'][$verifyfield]['error']) {
				if ("$var" != $this->form['fields'][$verifyfield]['value']) {
					$this->set_error($field);
				}
			}
		} else if ($crit_array[0] == "monoselect") {
			// to use this test specify
			// $section_form->add_element("moddate", "select", "monoselect|y=Yes:n=No");
			// $section_form->add_element("fieldname", "select", "monoselect|formvalue=ShowValue:n=No:other=Other thing");
			if (empty($var) AND !$this->form['fields'][$field]['mandatory']) {
				// if empty and not mandatory: do nothing..... ..

			} else if (!array_key_exists($var, $this->form['fields'][$field]['monoselect'])) {
				$this->form['fields'][$field]['value'] = $var;
				$this->set_error($field);
			} else {
				// check whichever one is "selected"
				$this->form['fields'][$field]['monoselect'][$var]['checked'] = "checked=\"checked\"";
				$this->form['fields'][$field]['monoselect'][$var]['selected'] = "selected=\"selected\"";
			}

		} else if ($crit_array[0] == "multiselect") {
			//echo "<code>$field, $var, $criteria</code>";
			// to use this test specify
			// $section_form->add_element("fieldname", "select", "multiselect|formvalue=ShowValue:n=No:other=Other thing");
			if (!is_array($var)) {
				$var_array = explode(":", $var);
			} else {
				$var_array = $var;
			}
			unset($var);
			unset($this->form['fields'][$field]['value']);
			foreach ($var_array as $var) {
				if (empty($var) AND !$this->form['fields'][$field]['mandatory']) {
					// if empty and not mandatory: do nothing..... ..

				} else if (!array_key_exists($var, $this->form['fields'][$field]['multiselect'])) {
					$this->form['fields'][$field]['value'] = $var;
					$this->set_error($field);
				} else {
					// check whichever one is "selected"
					$this->form['fields'][$field]['multiselect'][$var]['checked'] = "checked=\"checked\"";
					$this->form['fields'][$field]['multiselect'][$var]['selected'] = "selected=\"selected\"";

					$this->form['fields'][$field]['value'] .= "$var:";
				}
			}
			$this->form['fields'][$field]['value'] = rtrim($this->form['fields'][$field]['value'], ":");

		} else if ($crit_array[0] != "free") {
			// compose stuff
			$test = $crit_array[0];
			$regex = str_replace("REPLACE", $crit_array[1], $tests[$test]);

			//echo "<code>Test:$test Field: $field - Var:$var --> RegEx:$regex</code>";
			// now test it
			if (!preg_match("$regex", "$var")) {
				$this->set_error($field);
			} else if ($crit_array[0] == "bool") {
				// BOOL fix
				if ($var) {
					$this->form['fields'][$field]['checked'] = "checked=\"checked\"";
					$this->form['fields'][$field]['selected'] = "selected=\"selected\"";
				} else {
					$this->form['fields'][$field]['value'] = 1;
				}
			}
		}
	}
	function errors() {
		if ($this->form['errors'] > 0) {
			return true;
		}
	}
	function error_on($field) {
		if ($this->form['fields'][$field]['error']) {
			return true;
		}
	}
	function get_values($all=FALSE) {
		// $all is enabled to obtain display_only vars
		foreach ($this->form['fields'] as $k => $v) {
			// Ignore display_only vars IF $all is true (default)
			if ($v['display_only'] AND !$all) {
				// Don't return var
			} else {
				$result[$k] = $v['value'];
			}
			// Reset value for unchecked checkboxes
			if ($v['criteria'] == "bool") {
				$this->form['fields'][$k]['value'] = 1;
			}
		}
		return $result;
	}
	function get_value($field) {
		if ($this->form['fields'][$field]['criteria'] == "bool") {
			if ($this->form['fields'][$field]['checked']) {
				return true;
			} else {
				return false;
			}
		} else {
			return $this->form['fields'][$field]['value'];
		}
	}
	function delete_values() {
		foreach ($this->form['fields'] as $k => $v) {
			$this->form['fields'][$k]['value'] = '';
		}
	}
	function result() {
		return $this->form;
	}
}


class FormPrinter {
	public function __construct($form, $action="") {
		// override default action
		if (empty($action)) {
			$action = $form['form_action'];
		}
		// New method - send over the whole array so we can also fill in the action field properly
		if (is_array($form)) {
			$this->form_name = $form['form_name'];
			if (!empty($form['form_id'])) { $this->form_id = $form['form_id']; } else { $this->form_id = ''; }
			$this->form = '<form action="'.$action.'" method="post" enctype="multipart/form-data" class="generatedform">';
			if (is_array($form['fields'])) {
				$this->fields = $form['fields'];
			}
		// OLD method
		} else {
			$this->form_name = $form;
			$this->form = '<form action="'.$action.'" method="post" enctype="multipart/form-data" class="generatedform">';
		}
		$this->css['span'] = "hidden";
		$this->end_of_form = '';
	}
	function form_css_id($id) {
		$this->form = str_replace("<form", "<form id=\"$id\" ", $this->form);
	}
	function form_css_class($class) {
		$this->form = str_replace("generatedform", "generatedform $class", $this->form);
	}
	function configure_css($type, $class) {
		$this->css[$type] = $class;
	}
	function add_css_class($field, $class) {
		$this->fields[$field]['css'] .= " $class";
	}
	function add_placeholder($field, $placeholder) {
		$this->fields[$field]['placeholder'] = $placeholder;
	}
	// function to do it all
	function auto_generate_form($fields="") {
		if (!is_array($fields)) {
			$fields = $this->fields;
		}
		foreach ($fields as $k => $v) {
			$id = md5("$this->form_name.$this->form_id.$k");

			if (!empty($v['desc'])) {
				$this->desc = $v['desc'];
			} else {
				$this->desc = '';
			}

			if ($v['type'] == "hidden") {
				$this->hidden_field($v, $id);
			} else {

				$this->form .= '<div class="variable">';
				$this->form .= $this->label($v['name'], $v['display_name'], $id, $v['css']." main");
				$this->form .= '<div class="inputfield">';

				// In case of lists, get value from sub_array
				if (!empty($v['display_only'])) {
					if (is_array($v['monoselect'])) {
						$print_key = $v['value'];
						$print_value = $v['monoselect'][$print_key]['val'];
						$this->form .= '<span class="display-only">'.$print_value.'</span>';
					} else {
						$this->form .= '<span class="display-only">'.$v['value'].'</span>';
					}
					$this->hidden_field($v, $id);

				} else if ($v['type'] == "text") {
					$this->text_input($v, $id);
				} else if ($v['type'] == "textarea") {
					$this->text_textarea($v, $id);
				} else if ($v['type'] == "password") {
					$this->text_password($v, $id);
				} else if ($v['type'] == "select") {
					$this->select_input($v, $id);
				} else if ($v['type'] == "radio") {
					$this->radio_input($v, $id);
				} else if ($v['type'] == "checkbox") {
					$this->checkbox_input($v, $id);
				} else if ($v['type'] == "file") {
					$this->file_input($v, $id);
				}

				if (!empty($v['input_desc'])) {
					$this->form .= '<span>'.$v['input_desc'].'</span>';
				}

				$this->form .= "</div>";
				$this->form .= "</div>";
			}
		}
	}

	function insert_html($html, $end_of_form=0) {
		if ($end_of_form) {
			$this->end_of_form .= $html;
		} else {
			$this->form .= $html;
		}
	}
	function label($name, $display_name, $id="", $error_css="") {
		if (!empty($this->desc)) {
			$desc = ' <span class="'.$this->css['span'].'">'.$this->desc.'</span>';
		} else {
			$desc = '';
		}
		if (empty($display_name)) { // TODO check if !empty is required
			$display_name = $name;
		}
		if (empty($this->css['label'])) { $this->css['label'] = ''; }
		return '<div class="inputlabel"><label class="'.$this->css['label'].' '.$error_css.'" for="'.$id.'">'.$display_name.''.$desc.'</label></div>';
	}
	// Various field types
	function text_input($array, $id="") {
		$csslang = '';
		$placeholder = '';
		$maxlength = '';
		if (!empty($array['lang']) AND !empty($this->css['input'])) {
			$csslang = " ".$this->css['input']."_".$array['lang'];
		}
		if (!empty($array['placeholder'])) {
			$placeholder = 'placeholder="'.$array['placeholder'].'"';
		}
		if (!empty($array['maxlength'])) {
			$maxlength = 'maxlength="'.$array['maxlength'].'"';
		}
		if (empty($this->css['input'])) { $this->css['input'] = ''; }
		$this->form .= '<input class="'.$this->css['input'].''.$csslang.' '.$array['css'].'" type="text" name="'.$array['name'].'" id="'.$id.'" value="'.$array['value'].'" '.$placeholder.' '.$maxlength.'/>';
	}
	function file_input($array, $id="") { // FIXME this is not verified (yet)
		if ($array['lang'] AND !empty($this->css['input'])) {
			$csslang = " ".$this->css['input']."_".$array['lang']."";
		}
		$this->form .= '<input class="'.$this->css['input'].''.$csslang.' '.$array['css'].'" type="file" name="'.$array['name'].'" id="'.$id.'" value="'.$array['value'].'" />';
	}
	function hidden_field($array, $id="") {
		$this->form .= '<input type="hidden" name="'.$array['name'].'" value="'.$array['value'].'">';
	}
	function text_textarea($array, $id="") {
		if (!empty($array['lang']) AND !empty($this->css['textarea'])) {
			$csslang = " ".$this->css['textarea']."_".$array['lang'];
		} else {
			$csslang = '';
		}
		if (empty($this->css['textarea'])) { $this->css['textarea'] = ''; }
		$this->form .= '<textarea class="'.$this->css['textarea'].''.$csslang.' '.$array['css'].'" name="'.$array['name'].'" id="'.$id.'">'.$array['value'].'</textarea>';
	}
	function text_password($array, $id="") {
		if ($array['lang'] AND !empty($this->css['input'])) {
			$csslang = " ".$this->css['input']."_".$array['lang']."";
		}
		$this->form .= '<input class="'.$this->css['input'].''.$csslang.' '.$array['css'].'" type="password" name="'.$array['name'].'" id="'.$id.'" value="'.$array['value'].'" />';
	}
	function select_input($array, $id="") {
		if ($array['multiselect']) {
			$this->form .= '<select class="'.$this->css['select'].' '.$array['css'].'" name="'.$array['name'].'" id="'.$id.'" multiple="multiple">';
			$array['monoselect'] = $array['multiselect'];
		} else {
			$this->form .= '<select class="'.$this->css['select'].' '.$array['css'].'" name="'.$array['name'].'" id="'.$id.'">';
		}
		foreach ($array['monoselect'] as $k => $v) {
			$this->form .= '<option value="'.$k.'" '.$v['selected'].'>'.$v['val'].'</option>';
		}
		$this->form .= '</select>';
	}
	function radio_input($array, $id="") {
		$this->form .= '<div class="inputfield2">';
		foreach ($array['monoselect'] as $k => $v) {
			$this->form .= $this->label($v['val'], "", "$id".$v['val']."");
			$this->form .= '<input class="'.$this->css['radio'].'" type="radio" name="'.$array['name'].'" id="'.$id.$v['val'].'" value="'.$k.'" '.$v['checked'].' />';
		}
		$this->form .= '</div>';
	}

	function checkbox_input($array, $id="") {
		if (!empty($array['multiselect']) OR !empty($array['monoselect'])) {
			// Do the array way
			$this->form .= '<div class="inputfield2">';
			if (!empty($array['multiselect'])) {
				$this_array = $array['multiselect'];
			} else if (!empty($array['monoselect'])) {
				$this_array = $array['monoselect'];
			}
			foreach ($this_array as $k => $v) {
				$this->form .= $this->label($v['val'], "", "$id".$v['val']."");
				$this->form .= '<input class="'.$this->css['checkbox'].'" type="checkbox" name="'.$array['name'].'['.$k.']" id="'.$id.$v['val'].'" value="'.$k.'" '.$v['checked'].' />';
			}
		} else {
			// Fix the god damn unset vars !
			if (empty($array['checked'])) { $array['checked'] = ''; }
			if (empty($this->css['checkbox'])) { $this->css['checkbox'] = ''; }
			// Do the mono rail way
			$this->form .= '<div class="inputfield2">';
			$this->form .= '<input class="'.$this->css['checkbox'].'" type="checkbox" name="'.$array['name'].'" id="'.$id.'" value="'.$array['value'].'" '.$array['checked'].' />';
		}
		$this->form .= '</div>';
	}

	function print_form($submit_txt) {
		$this->form .= '<input class="submit '.$this->css['submit'].'" type="submit" name="'.$this->form_name.'" value="'.$submit_txt.'" />'.$this->end_of_form.'</form>';
		echo $this->form;
	}
	function return_form($submit_txt) {
		$this->form .= '<input class="submit '.$this->css['submit'].'" type="submit" name="'.$this->form_name.'" value="'.$submit_txt.'" />'.$this->end_of_form.'</form>';
		return $this->form;
	}
}
?>
