/*
Mailboy - A Php/MySQL application that offers a web based interface
to administer an email server. It can be used by the server administrator
as well as the email users.
(c) 2013 Emmanuel Revah - manu-at-manurevah.com
http://mailboy.manurevah.com/

This file is part of Mailboy.

Mailboy is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Mailboy is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Mailboy.  If not, see <http://www.gnu.org/licenses/>.
*/

CREATE DATABASE IF NOT EXISTS `mailboy`;
USE mailboy;

CREATE TABLE IF NOT EXISTS `mailboy_cfg` (
	`key` varchar(16) NOT NULL,
	`value` VARCHAR(256) NOT NULL,
	UNIQUE KEY `key` (`key`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
INSERT INTO `mailboy_cfg` (`key`, `value`) VALUES ('db_version', '6');


CREATE TABLE IF NOT EXISTS `virtual_domains` (
	id INT(11) UNSIGNED NOT NULL AUTO_INCREMENT,
	name VARCHAR(64) NOT NULL,
	default_policy INT(11) NOT NULL,
	default_quota_kb INT(11) NOT NULL DEFAULT '0',
	default_xmpp TINYINT( 1 ) UNSIGNED NOT NULL DEFAULT '0',
	default_mail TINYINT( 1 ) UNSIGNED NOT NULL DEFAULT '1',
	ml TINYINT( 1 ) UNSIGNED NOT NULL DEFAULT '0',
	PRIMARY KEY (`id`),
	UNIQUE KEY `name` (`name`)
) ENGINE=InnoDB  DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `virtual_users` (
	id INT(11) UNSIGNED NOT NULL AUTO_INCREMENT,
	domain_id INT(11) UNSIGNED NOT NULL,
	user VARCHAR(64) NOT NULL,
	password VARCHAR(255) NOT NULL,
	quota_kb INT(11) NOT NULL DEFAULT '0',
	quota_messages INT(11) NOT NULL DEFAULT '0',
	policy_id INT(11) UNSIGNED NOT NULL,
	admin TINYINT(1)  UNSIGNED NOT NULL DEFAULT '0',
	xmpp TINYINT(1)  UNSIGNED NOT NULL DEFAULT '0',
	mail TINYINT(1)  UNSIGNED NOT NULL DEFAULT '1',
	suspend TINYINT(1)  UNSIGNED NOT NULL DEFAULT '0',
	PRIMARY KEY (`id`),
	UNIQUE key `account` (`domain_id`,`user`),
	FOREIGN KEY (`domain_id`) REFERENCES `virtual_domains`(`id`) ON DELETE CASCADE
) ENGINE=InnoDB  DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `virtual_aliases` (
	id INT(11) UNSIGNED NOT NULL AUTO_INCREMENT,
	domain_id INT(11) UNSIGNED NOT NULL,
	source VARCHAR(64) NOT NULL,
	policy_id INT(11) UNSIGNED NOT NULL,
	suspend TINYINT(1)  UNSIGNED NOT NULL DEFAULT '0',
	PRIMARY KEY (`id`),
	UNIQUE KEY `alias` (`domain_id`,`source`),
	FOREIGN KEY (`domain_id`) REFERENCES `virtual_domains`(`id`) ON DELETE CASCADE
) ENGINE=InnoDB  DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `virtual_alias_destinations` (
	id INT(11) UNSIGNED NOT NULL AUTO_INCREMENT,
	alias_id INT(11) UNSIGNED NOT NULL,
	destination VARCHAR(127) NOT NULL,
	PRIMARY KEY (`id`),
	UNIQUE key `alias` (`alias_id`,`destination`),
	FOREIGN KEY (`alias_id`) REFERENCES `virtual_aliases`(`id`) ON DELETE CASCADE
) ENGINE=InnoDB  DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `domain_admin_assoc` (
    `user_id` INT(11) UNSIGNED NOT NULL,
    `domain_id` INT(11) UNSIGNED NOT NULL,
    PRIMARY KEY (`user_id`, `domain_id`),
    FOREIGN KEY (`user_id`) REFERENCES `virtual_users`(`id`) ON DELETE CASCADE,
    FOREIGN KEY (`domain_id`) REFERENCES `virtual_domains`(`id`) ON DELETE CASCADE
) ENGINE=InnoDB  DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `relay_domains` (
	`id` INT(11) NOT NULL AUTO_INCREMENT,
	`domain` VARCHAR(50) NOT NULL,
	`note` text NOT NULL,
	PRIMARY KEY  (`id`),
	UNIQUE KEY `domain` (`domain`)
) ENGINE=InnoDB  DEFAULT CHARSET=utf8;

INSERT INTO `relay_domains` (`id`, `domain`, `note`) VALUES
(1, '$mydestination', '');

/* POLICY */
CREATE TABLE IF NOT EXISTS `policy` (
	`id` INT(11) UNSIGNED NOT NULL AUTO_INCREMENT,
	`policy_name` VARCHAR(255) DEFAULT NULL,
	`virus_lover` ENUM('Y', 'N') DEFAULT 'N' NOT NULL,
	`spam_lover` ENUM('Y', 'N') DEFAULT 'N' NOT NULL,
	`banned_files_lover` ENUM('Y', 'N') DEFAULT 'N' NOT NULL,
	`bad_header_lover` ENUM('Y', 'N') DEFAULT 'N' NOT NULL,
	`bypass_virus_checks` ENUM('Y', 'N') DEFAULT 'N' NOT NULL,
	`bypass_spam_checks` ENUM('Y', 'N') DEFAULT 'N' NOT NULL,
	`bypass_banned_checks` ENUM('Y', 'N') DEFAULT 'N' NOT NULL,
	`bypass_header_checks` ENUM('Y', 'N') DEFAULT 'N' NOT NULL,
	`spam_quarantine_to` VARCHAR(128) DEFAULT NULL,
	`spam_tag_level` FLOAT DEFAULT '999',
	`spam_tag2_level` FLOAT DEFAULT '999',
	`spam_kill_level` FLOAT DEFAULT '999',
	`spam_subject_tag2` VARCHAR(64) DEFAULT NULL,
	PRIMARY KEY (`id`),
	UNIQUE KEY `policy_name` (`policy_name`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

/* Add default policies */
INSERT INTO policy VALUES (
1,'anti virus/spam','N','N','N','N','N','N','N','N','', -1000, 5, 10,'[SPAM] '),(
2,'anti virus/spam - no subject change','N','N','N','N','N','N','N','N','',-1000,5,10,''),(
3,'no scan','Y','Y','Y','Y','Y','Y','Y','Y','',10000, 10000, 10000, '');

/* Sender ACLs */
CREATE TABLE IF NOT EXISTS `ip_access` (
	`id` INT(10) NOT NULL AUTO_INCREMENT,
	`ip` VARCHAR(15) NOT NULL,
	`state` ENUM('OK','REJECT') NOT NULL,
	`note` TEXT NOT NULL,
	`bounce_message` TEXT NOT NULL,
	PRIMARY KEY (`id`),
	UNIQUE KEY `ip` (`ip`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
CREATE TABLE IF NOT EXISTS `domain_access` (
	`id` INT(10) NOT NULL AUTO_INCREMENT,
	`domain` VARCHAR(255) NOT NULL,
	`state` ENUM('OK','REJECT') NOT NULL,
	`note` TEXT NOT NULL,
	`bounce_message` TEXT NOT NULL,
	PRIMARY KEY (`id`),
	UNIQUE KEY `ip` (`domain`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;


/* VIEWS */
CREATE VIEW view_users AS
SELECT virtual_users.id AS uid, user, CONCAT(user, '@', name) AS email, password,
quota_kb, quota_messages, name AS domain, virtual_domains.id AS domain_id, policy_name, policy_id, admin, xmpp, mail, suspend,
GROUP_CONCAT(DISTINCT domain_admin_assoc.domain_id) AS admin_of_ids,
GROUP_CONCAT((SELECT virtual_domains.name FROM virtual_domains WHERE domain_admin_assoc.domain_id = virtual_domains.id)) AS admin_of_names
FROM virtual_users
LEFT JOIN virtual_domains ON (virtual_users.domain_id = virtual_domains.id)
LEFT JOIN policy ON (virtual_users.policy_id = policy.id)
LEFT JOIN domain_admin_assoc ON (virtual_users.id = domain_admin_assoc.user_id)
GROUP BY virtual_users.id;

/* VIEW ALIASES - new and old */
CREATE VIEW view_aliases AS
SELECT virtual_aliases.id, domain_id, source, name, CONCAT(source, '@', name) AS email, policy_id,
virtual_alias_destinations.destination
FROM virtual_aliases
LEFT JOIN virtual_domains ON (virtual_aliases.domain_id = virtual_domains.id)
RIGHT JOIN virtual_alias_destinations ON (virtual_aliases.id = virtual_alias_destinations.alias_id);

CREATE VIEW view_domains AS
SELECT virtual_domains.id, virtual_domains.name, virtual_domains.default_policy, policy_name, policy.id AS policy_id,
virtual_domains.default_quota_kb AS default_quota, virtual_domains.default_mail AS default_mail, virtual_domains.default_xmpp AS default_xmpp, virtual_domains.ml AS ml,
(SELECT count(virtual_users.id) FROM virtual_users WHERE virtual_users.domain_id = virtual_domains.id) AS accounts,
(SELECT count(virtual_aliases.id) FROM virtual_aliases WHERE virtual_aliases.domain_id = virtual_domains.id) AS aliases
FROM virtual_domains
LEFT JOIN policy ON (virtual_domains.default_policy = policy.id);


/* Optimised `views` for use by mail server bits */
CREATE VIEW view_mail_users AS
SELECT user, CONCAT(user, '@', name) AS email, password, quota_kb, quota_messages, policy_id
FROM virtual_users
LEFT JOIN virtual_domains ON (virtual_users.domain_id = virtual_domains.id)
WHERE `mail` =1 AND `suspend` =0;

/* We can now view users and aliases ! */
CREATE VIEW view_users_policies AS
SELECT email, virus_lover, spam_lover, banned_files_lover, bad_header_lover, bypass_virus_checks, bypass_spam_checks,
bypass_banned_checks, bypass_header_checks, spam_quarantine_to,
spam_tag_level, spam_tag2_level, spam_kill_level, spam_subject_tag2
FROM view_mail_users, policy
WHERE (view_mail_users.policy_id = policy.id)
UNION ALL
SELECT email, virus_lover, spam_lover, banned_files_lover, bad_header_lover, bypass_virus_checks, bypass_spam_checks,
bypass_banned_checks, bypass_header_checks, spam_quarantine_to,
spam_tag_level, spam_tag2_level, spam_kill_level, spam_subject_tag2
FROM view_aliases, policy
WHERE (view_aliases.policy_id = policy.id);

CREATE VIEW view_xmpp_users AS
SELECT CONCAT(user, '@', name) AS email, password
FROM virtual_users
LEFT JOIN virtual_domains ON (virtual_users.domain_id = virtual_domains.id)
WHERE `xmpp` =1 AND `suspend` =0;


/* Mailing lists - Experimental */
CREATE TABLE IF NOT EXISTS `virtual_mailing_lists` (
	id INT(11) UNSIGNED NOT NULL AUTO_INCREMENT,
	domain_id INT(11) UNSIGNED NOT NULL,
	ml VARCHAR(64) NOT NULL,
	transport VARCHAR(127) NOT NULL,
	PRIMARY KEY (`id`),
	UNIQUE KEY `ml` (`domain_id`,`ml`),
	FOREIGN KEY (`domain_id`) REFERENCES `virtual_domains` (`id`) ON DELETE CASCADE
) ENGINE=InnoDB  DEFAULT CHARSET=utf8;

CREATE VIEW view_mailing_lists AS
SELECT virtual_mailing_lists.id, domain_id, `virtual_mailing_lists`.`ml`, name as domain,
CONCAT(`virtual_mailing_lists`.`ml`, '@', name) AS mailing_list, transport
FROM virtual_mailing_lists LEFT JOIN virtual_domains ON (virtual_mailing_lists.domain_id = virtual_domains.id)
